@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
@@ -323,13 +323,13 @@ gdm_server_init_command (GdmServer *server)
                 goto fallback;
         }
 
-        server->priv->command = g_strdup_printf (SYSTEMD_X_SERVER " -br -verbose%s", debug_options);
+        server->priv->command = g_strdup_printf (SYSTEMD_X_SERVER " -background none -verbose%s", debug_options);
         return;
 
 fallback:
 #endif
 
-        server->priv->command = g_strdup_printf (X_SERVER " -br -verbose%s", debug_options);
+        server->priv->command = g_strdup_printf (X_SERVER " -background none -verbose%s", debug_options);
 }
 
 static gboolean
@@ -722,7 +722,7 @@ gdm_server_spawn (GdmServer  *server,
                 g_warning (_("%s: Empty server command for display %s"),
                            "gdm_server_spawn",
                            server->priv->display_name);
-                _exit (SERVER_ABORT);
+                goto out;
         }
 
         env = get_server_environment (server);
@@ -755,12 +755,15 @@ gdm_server_spawn (GdmServer  *server,
         g_ptr_array_foreach (env, (GFunc)g_free, NULL);
         g_ptr_array_free (env, TRUE);
 
-        g_debug ("GdmServer: Started X server process %d - waiting for READY", (int)server->priv->pid);
+        if (ret) {
+                g_debug ("GdmServer: Started X server process %d - waiting for READY", (int)server->priv->pid);
 
-        server->priv->child_watch_id = g_child_watch_add (server->priv->pid,
-                                                          (GChildWatchFunc)server_child_watch,
-                                                          server);
+                server->priv->child_watch_id = g_child_watch_add (server->priv->pid,
+                                                                  (GChildWatchFunc)server_child_watch,
+                                                                  server);
+        }
 
+out:
         return ret;
 }
 
